@@ -18,7 +18,7 @@
 #define _ESP8266_COMPAT_H_
 
 #define ICACHE_FLASH_ATTR
-#define ICACHE_RAM_ATTR IRAM_ATTR
+#define ICACHE_RAM_ATTR ARDUINO_ISR_ATTR
 
 
 #endif /* _ESP8266_COMPAT_H_ */
\ No newline at end of file
