@@ -150,7 +150,7 @@ static bool _udp_task_start(){
         }
     }
     if(!_udp_task_handle){
-        xTaskCreateUniversal(_udp_task, "async_udp", 4096, NULL, 3, (TaskHandle_t*)&_udp_task_handle, CONFIG_ARDUINO_UDP_RUNNING_CORE);
+        xTaskCreateUniversal(_udp_task, "async_udp", 4096, NULL, CONFIG_ARDUINO_UDP_TASK_PRIORITY, (TaskHandle_t*)&_udp_task_handle, CONFIG_ARDUINO_UDP_RUNNING_CORE);
         if(!_udp_task_handle){
             return false;
         }
@@ -277,6 +277,22 @@ void AsyncUDPMessage::flush()
     _index = 0;
 }
 
+AsyncUDPPacket::AsyncUDPPacket(AsyncUDPPacket &packet){
+    _udp = packet._udp;
+    _pb = packet._pb;
+    _if = packet._if;
+    _data = packet._data;
+    _len = packet._len;
+    _index = 0;
+
+    memcpy(&_remoteIp, &packet._remoteIp, sizeof(ip_addr_t));
+    memcpy(&_localIp, &packet._localIp, sizeof(ip_addr_t));
+    _localPort = packet._localPort;
+    _remotePort = packet._remotePort;
+    memcpy(_remoteMac, packet._remoteMac, 6);
+
+    pbuf_ref(_pb);
+}
 
 AsyncUDPPacket::AsyncUDPPacket(AsyncUDP *udp, pbuf *pb, const ip_addr_t *raddr, uint16_t rport, struct netif * ntif)
 {
@@ -294,18 +310,18 @@ AsyncUDPPacket::AsyncUDPPacket(AsyncUDP *udp, pbuf *pb, const ip_addr_t *raddr,
     _localIp.type = _remoteIp.type;
 
     eth_hdr* eth = NULL;
-    udp_hdr* udphdr = reinterpret_cast<udp_hdr*>(_data - UDP_HLEN);
+    udp_hdr* udphdr = (udp_hdr *)(_data - UDP_HLEN);
     _localPort = ntohs(udphdr->dest);
     _remotePort = ntohs(udphdr->src);
 
     if (_remoteIp.type == IPADDR_TYPE_V4) {
-        eth = (eth_hdr *)(((uint8_t *)(pb->payload)) - UDP_HLEN - IP_HLEN - SIZEOF_ETH_HDR);
-        struct ip_hdr * iphdr = (struct ip_hdr *)(((uint8_t *)(pb->payload)) - UDP_HLEN - IP_HLEN);
+        eth = (eth_hdr *)(_data - UDP_HLEN - IP_HLEN - SIZEOF_ETH_HDR);
+        struct ip_hdr * iphdr = (struct ip_hdr *)(_data - UDP_HLEN - IP_HLEN);
         _localIp.u_addr.ip4.addr = iphdr->dest.addr;
         _remoteIp.u_addr.ip4.addr = iphdr->src.addr;
     } else {
-        eth = (eth_hdr *)(((uint8_t *)(pb->payload)) - UDP_HLEN - IP6_HLEN - SIZEOF_ETH_HDR);
-        struct ip6_hdr * ip6hdr = (struct ip6_hdr *)(((uint8_t *)(pb->payload)) - UDP_HLEN - IP6_HLEN);
+        eth = (eth_hdr *)(_data - UDP_HLEN - IP6_HLEN - SIZEOF_ETH_HDR);
+        struct ip6_hdr * ip6hdr = (struct ip6_hdr *)(_data - UDP_HLEN - IP6_HLEN);
         memcpy(&_localIp.u_addr.ip6.addr, (uint8_t *)ip6hdr->dest.addr, 16);
         memcpy(&_remoteIp.u_addr.ip6.addr, (uint8_t *)ip6hdr->src.addr, 16);
     }
@@ -479,6 +495,7 @@ AsyncUDP::AsyncUDP()
 {
     _pcb = NULL;
     _connected = false;
+	_lastErr = ERR_OK;
     _handler = NULL;
 }
 
@@ -517,8 +534,8 @@ bool AsyncUDP::connect(const ip_addr_t *addr, uint16_t port)
     }
     close();
     UDP_MUTEX_LOCK();
-    err_t err = _udp_connect(_pcb, addr, port);
-    if(err != ERR_OK) {
+    _lastErr = _udp_connect(_pcb, addr, port);
+    if(_lastErr != ERR_OK) {
         UDP_MUTEX_UNLOCK();
         return false;
     }
@@ -646,7 +663,7 @@ size_t AsyncUDP::writeTo(const uint8_t * data, size_t len, const ip_addr_t * add
     if(len > CONFIG_TCP_MSS) {
         len = CONFIG_TCP_MSS;
     }
-    err_t err = ERR_OK;
+    _lastErr = ERR_OK;
     pbuf* pbt = pbuf_alloc(PBUF_TRANSPORT, len, PBUF_RAM);
     if(pbt != NULL) {
         uint8_t* dst = reinterpret_cast<uint8_t*>(pbt->payload);
@@ -656,16 +673,16 @@ size_t AsyncUDP::writeTo(const uint8_t * data, size_t len, const ip_addr_t * add
             void * nif = NULL;
             tcpip_adapter_get_netif((tcpip_adapter_if_t)tcpip_if, &nif);
             if(!nif){
-                err = _udp_sendto(_pcb, pbt, addr, port);
+                _lastErr = _udp_sendto(_pcb, pbt, addr, port);
             } else {
-                err = _udp_sendto_if(_pcb, pbt, addr, port, (struct netif *)nif);
+                _lastErr = _udp_sendto_if(_pcb, pbt, addr, port, (struct netif *)nif);
             }
         } else {
-            err = _udp_sendto(_pcb, pbt, addr, port);
+            _lastErr = _udp_sendto(_pcb, pbt, addr, port);
         }
         UDP_MUTEX_UNLOCK();
         pbuf_free(pbt);
-        if(err < ERR_OK) {
+        if(_lastErr < ERR_OK) {
             return 0;
         }
         return len;
@@ -682,9 +699,8 @@ void AsyncUDP::_recv(udp_pcb *upcb, pbuf *pb, const ip_addr_t *addr, uint16_t po
         if(_handler) {
             AsyncUDPPacket packet(this, this_pb, addr, port, netif);
             _handler(packet);
-        } else {
-            pbuf_free(this_pb);
         }
+        pbuf_free(this_pb);
     }
 }
 
@@ -870,6 +886,10 @@ bool AsyncUDP::connected()
     return _connected;
 }
 
+esp_err_t AsyncUDP::lastErr() {
+	return _lastErr;
+}
+
 void AsyncUDP::onPacket(AuPacketHandlerFunctionWithArg cb, void * arg)
 {
     onPacket(std::bind(cb, arg, std::placeholders::_1));
