@@ -4,9 +4,9 @@
 #include "IPAddress.h"
 #include "IPv6Address.h"
 #include "Print.h"
+#include "Stream.h"
 #include <functional>
 extern "C" {
-#include "lwip/ip_addr.h"
 #include "esp_netif.h"
 #include "freertos/queue.h"
 #include "freertos/semphr.h"
