@@ -23,11 +23,15 @@ class SPIFFSFS : public FS
 {
 public:
     SPIFFSFS();
-    bool begin(bool formatOnFail=false, const char * basePath="/spiffs", uint8_t maxOpenFiles=10);
+    ~SPIFFSFS();
+    bool begin(bool formatOnFail=false, const char * basePath="/spiffs", uint8_t maxOpenFiles=10, const char * partitionLabel=NULL);
     bool format();
     size_t totalBytes();
     size_t usedBytes();
     void end();
+
+private:
+    char * partitionLabel_;
 };
 
 }
