@@ -1,3 +1,5 @@
+#ifndef HTPSPOTUADATE_H
+#define HTPSPOTUADATE_H
 #include "esp_http_client.h"
 #define HttpEvent_t esp_http_client_event_t
 
@@ -19,3 +21,4 @@ class HttpsOTAUpdateClass {
 };
 
 extern HttpsOTAUpdateClass HttpsOTA;
+#endif
