@@ -11,6 +11,9 @@
 extern "C" {
 #endif
 
+//This is the ADC calibration value version burnt in efuse
+#define ESP_EFUSE_ADC_CALIB_VER     1
+
 /**
  * @brief Get the RTC calibration efuse version
  *
@@ -21,11 +24,12 @@ int esp_efuse_rtc_calib_get_ver(void);
 /**
  * @brief Get the init code in the efuse, for the corresponding attenuation.
  *
- * @param version Version of the stored efuse
- * @param atten  Attenuation of the init code
+ * @param version   Version of the stored efuse
+ * @param adc_unit  ADC unit. Not used, for compatibility. On esp32c3, for calibration v1, both ADC units use the same init code (calibrated by ADC1)
+ * @param atten     Attenuation of the init code
  * @return The init code stored in efuse
  */
-uint16_t esp_efuse_rtc_calib_get_init_code(int version, int atten);
+uint32_t esp_efuse_rtc_calib_get_init_code(int version, uint32_t adc_unit, int atten);
 
 /**
  * @brief Get the calibration digits stored in the efuse, and the corresponding voltage.
