@@ -73,20 +73,18 @@ typedef volatile struct efuse_dev_s {
     } pgm_data3;
     union {
         struct {
-            uint32_t dis_download_mode:        1;                /*Set this bit to disable download mode (boot_mode[3:0] = 0  1  2  3  6  7).*/
-            uint32_t dis_legacy_spi_boot:      1;                /*Set this bit to disable Legacy SPI boot mode (boot_mode[3:0] = 4).*/
-            uint32_t uart_print_channel:       1;                /*Selectes the default UART print channel. 0: UART0. 1: UART1.*/
-            uint32_t flash_ecc_mode:           1;                /*Set ECC mode in ROM  0: ROM would Enable Flash ECC 16to18 byte mode. 1:ROM would use 16to17 byte mode.*/
-            uint32_t dis_usb_download_mode:    1;                /*Set this bit to disable UART download mode through USB.*/
-            uint32_t enable_security_download: 1;                /*Set this bit to enable secure UART download mode.*/
-            uint32_t uart_print_control:       2;                /*Set the default UARTboot message output mode. 00: Enabled. 01: Enabled when GPIO8 is low at reset. 10: Enabled when GPIO8 is high at reset. 11:disabled.*/
-            uint32_t pin_power_selection:      1;                /*GPIO33-GPIO37 power supply selection in ROM code. 0: VDD3P3_CPU. 1: VDD_SPI.*/
-            uint32_t flash_type:               1;                /*Set the maximum lines of SPI flash. 0: four lines. 1: eight lines.*/
-            uint32_t flash_page_size:          2;                /*Set Flash page size.*/
-            uint32_t flash_ecc_en:             1;                /*Set 1 to enable ECC for flash boot.*/
-            uint32_t force_send_resume:        1;                /*Set this bit to force ROM code to send a resume command during SPI boot.*/
-            uint32_t secure_version:          16;                /*Secure version (used by ESP-IDF anti-rollback feature).*/
-            uint32_t rpt4_reserved1:           2;                /*Reserved (used for four backups method).*/
+            uint32_t dis_download_mode:                 1;                /*Set this bit to disable download mode (boot_mode[3:0] = 0  1  2  3  6  7).*/
+            uint32_t dis_direct_boot:                   1;                /*Set this bit to disable direct boot.*/
+            uint32_t dis_usb_serial_jtag_rom_print:     1;                /*Set this bit to disable USB-Serial-JTAG print during rom boot*/
+            uint32_t rpt4_reserved8:                    1;                /*Reserved (used for four backups method).*/
+            uint32_t dis_usb_serial_jtag_download_mode: 1;                /*Set this bit to disable download mode through USB-Serial-JTAG.*/
+            uint32_t enable_security_download:          1;                /*Set this bit to enable secure UART download mode.*/
+            uint32_t uart_print_control:                2;                /*Set the default UARTboot message output mode. 00: Enabled. 01: Enabled when GPIO8 is low at reset. 10: Enabled when GPIO8 is high at reset. 11:disabled.*/
+            uint32_t rpt4_reserved7:                    5;                /*Reserved (used for four backups method).*/
+            uint32_t force_send_resume:                 1;                /*Set this bit to force ROM code to send a resume command during SPI boot.*/
+            uint32_t secure_version:                    16;                /*Secure version (used by ESP-IDF anti-rollback feature).*/
+            uint32_t rpt4_reserved1:                    1;                /*Reserved (used for four backups method).*/
+            uint32_t err_rst_enable:                    1;                /*Use BLOCK0 to check error record registers, 0 - without check.*/
         };
         uint32_t val;
     } pgm_data4;
@@ -158,19 +156,17 @@ typedef volatile struct efuse_dev_s {
     union {
         struct {
             uint32_t dis_download_mode:        1;                /*The value of DIS_DOWNLOAD_MODE.*/
-            uint32_t dis_legacy_spi_boot:      1;                /*The value of DIS_LEGACY_SPI_BOOT.*/
-            uint32_t uart_print_channel:       1;                /*The value of UART_PRINT_CHANNEL.*/
-            uint32_t flash_ecc_mode:           1;                /*The value of FLASH_ECC_MODE.*/
-            uint32_t dis_usb_download_mode:    1;                /*The value of DIS_USB_DOWNLOAD_MODE.*/
+            uint32_t dis_direct_boot:          1;                /*The value of DIS_DIRECT_BOOT.*/
+            uint32_t dis_usb_serial_jtag_rom_print:1;                /*The value of DIS_USB_SERIAL_JTAG_ROM_PRINT.*/
+            uint32_t rpt4_reserved8:           1;                /*Reserved.*/
+            uint32_t dis_usb_serial_jtag_download_mode:    1;    /*The value of dis_usb_serial_jtag_download_mode.*/
             uint32_t enable_security_download: 1;                /*The value of ENABLE_SECURITY_DOWNLOAD.*/
             uint32_t uart_print_control:       2;                /*The value of UART_PRINT_CONTROL.*/
-            uint32_t pin_power_selection:      1;                /*The value of PIN_POWER_SELECTION.*/
-            uint32_t flash_type:               1;                /*The value of FLASH_TYPE.*/
-            uint32_t flash_page_size:          2;                /*The value of FLASH_PAGE_SIZE.*/
-            uint32_t flash_ecc_en:             1;                /*The value of FLASH_ECC_EN.*/
+            uint32_t rpt4_reserved7:           5;                /*Reserved.*/
             uint32_t force_send_resume:        1;                /*The value of FORCE_SEND_RESUME.*/
             uint32_t secure_version:          16;                /*The value of SECURE_VERSION.*/
-            uint32_t rpt4_reserved1:           2;                /*Reserved.*/
+            uint32_t rpt4_reserved1:           1;                /*Reserved.*/
+            uint32_t err_rst_enable:           1;                /*Use BLOCK0 to check error record registers, 0 - without check.*/
         };
         uint32_t val;
     } rd_repeat_data3;
@@ -325,20 +321,18 @@ typedef volatile struct efuse_dev_s {
     } rd_repeat_err2;
     union {
         struct {
-            uint32_t dis_download_mode_err:        1;            /*If DIS_DOWNLOAD_MODE is 1  then it indicates a programming error.*/
-            uint32_t dis_legacy_spi_boot_err:      1;            /*If DIS_LEGACY_SPI_BOOT is 1  then it indicates a programming error.*/
-            uint32_t uart_print_channel_err:       1;            /*If UART_PRINT_CHANNEL is 1  then it indicates a programming error.*/
-            uint32_t flash_ecc_mode_err:           1;            /*If FLASH_ECC_MODE is 1  then it indicates a programming error.*/
-            uint32_t dis_usb_download_mode_err:    1;            /*If DIS_USB_DOWNLOAD_MODE is 1  then it indicates a programming error.*/
-            uint32_t enable_security_download_err: 1;            /*If ENABLE_SECURITY_DOWNLOAD is 1  then it indicates a programming error.*/
-            uint32_t uart_print_control_err:       2;            /*If any bit in UART_PRINT_CONTROL is 1  then it indicates a programming error.*/
-            uint32_t pin_power_selection_err:      1;            /*If PIN_POWER_SELECTION is 1  then it indicates a programming error.*/
-            uint32_t flash_type_err:               1;            /*If FLASH_TYPE is 1  then it indicates a programming error.*/
-            uint32_t flash_page_size:              2;            /*If any bits in FLASH_PAGE_SIZE is 1  then it indicates a programming error.*/
-            uint32_t flash_ecc_en:                 1;            /*If FLASH_ECC_EN_ERR is 1  then it indicates a programming error.*/
-            uint32_t force_send_resume_err:        1;            /*If FORCE_SEND_RESUME is 1  then it indicates a programming error.*/
-            uint32_t secure_version_err:          16;            /*If any bit in SECURE_VERSION is 1  then it indicates a programming error.*/
-            uint32_t rpt4_reserved1_err:           2;            /*Reserved.*/
+            uint32_t dis_download_mode_err:        1;            /*If the value is not zero then it indicates a programming error on DIS_DOWNLOAD_MODE.*/
+            uint32_t dis_direct_boot_err:          1;            /*If the value is not zero then it indicates a programming error on DIS_DIRECT_BOOT.*/
+            uint32_t dis_usb_serial_jtag_rom_print_err:1;            /*If the value is not zero then it indicates a programming error on DIS_USB_SERIAL_JTAG_ROM_PRINT.*/
+            uint32_t rpt4_reserved8_err:           1;            /*Reserved.*/
+            uint32_t dis_usb_serial_jtag_download_mode_err:    1; /*If the value is not zero then it indicates a programming error on DIS_USB_SERIAL_JTAG_DOWNLOAD_MODE.*/
+            uint32_t enable_security_download_err: 1;            /*If the value is not zero then it indicates a programming error on ENABLE_SECURITY_DOWNLOAD.*/
+            uint32_t uart_print_control_err:       2;            /*If the value is not zero  then it indicates a programming error on UART_PRINT_CONTROL.*/
+            uint32_t rpt4_reserved7_err:           5;            /*Reserved*/
+            uint32_t force_send_resume_err:        1;            /*If the value is not zero then it indicates a programming error on FORCE_SEND_RESUME.*/
+            uint32_t secure_version_err:          16;            /*If the value is not zero then it indicates a programming error on SECURE_VERSION.*/
+            uint32_t rpt4_reserved1_err:           1;            /*Reserved.*/
+            uint32_t err_rst_enable_err:           1;            /*Use BLOCK0 to check error record registers, 0 - without check.*/
         };
         uint32_t val;
     } rd_repeat_err3;
@@ -364,30 +358,30 @@ typedef volatile struct efuse_dev_s {
     union {
         struct {
             uint32_t mac_spi_8m_err_num: 3;                      /*The value of this signal means the number of error bytes.*/
-            uint32_t mac_spi_8m_fail:    1;                      /*0: Means no failure and that the data of MAC_SPI_8M is reliable 1: Means that programming user data failed and the number of error bytes is over 6.*/
+            uint32_t reserved3:          1;                      /*Reserved.*/
             uint32_t sys_part1_num:      3;                      /*The value of this signal means the number of error bytes.*/
-            uint32_t sys_part1_fail:     1;                      /*0: Means no failure and that the data of system part1 is reliable 1: Means that programming user data failed and the number of error bytes is over 6.*/
+            uint32_t mac_spi_8m_fail:    1;                      /*0: Means no failure and that the data of MAC_SPI_8M is reliable 1: Means that programming MAC_SPI_8M failed and the number of error bytes is over 6.*/
             uint32_t usr_data_err_num:   3;                      /*The value of this signal means the number of error bytes.*/
-            uint32_t usr_data_fail:      1;                      /*0: Means no failure and that the user data is reliable 1: Means that programming user data failed and the number of error bytes is over 6.*/
+            uint32_t sys_part1_fail:     1;                      /*0: Means no failure and that the data of system part1 is reliable 1: Means that programming the data of system part1 failed and the number of error bytes is over 6.*/
             uint32_t key0_err_num:       3;                      /*The value of this signal means the number of error bytes.*/
-            uint32_t key0_fail:          1;                      /*0: Means no failure and that the data of key$n is reliable 1: Means that programming key$n failed and the number of error bytes is over 6.*/
+            uint32_t usr_data_fail:      1;                      /*0: Means no failure and that the data of user data is reliable 1: Means that programming user data failed and the number of error bytes is over 6.*/
             uint32_t key1_err_num:       3;                      /*The value of this signal means the number of error bytes.*/
-            uint32_t key1_fail:          1;                      /*0: Means no failure and that the data of key$n is reliable 1: Means that programming key$n failed and the number of error bytes is over 6.*/
+            uint32_t key0_fail:          1;                      /*0: Means no failure and that the data of key0 is reliable 1: Means that programming key0 failed and the number of error bytes is over 6.*/
             uint32_t key2_err_num:       3;                      /*The value of this signal means the number of error bytes.*/
-            uint32_t key2_fail:          1;                      /*0: Means no failure and that the data of key$n is reliable 1: Means that programming key$n failed and the number of error bytes is over 6.*/
+            uint32_t key1_fail:          1;                      /*0: Means no failure and that the data of key1 is reliable 1: Means that programming key1 failed and the number of error bytes is over 6.*/
             uint32_t key3_err_num:       3;                      /*The value of this signal means the number of error bytes.*/
-            uint32_t key3_fail:          1;                      /*0: Means no failure and that the data of key$n is reliable 1: Means that programming key$n failed and the number of error bytes is over 6.*/
+            uint32_t key2_fail:          1;                      /*0: Means no failure and that the data of key2 is reliable 1: Means that programming key2 failed and the number of error bytes is over 6.*/
             uint32_t key4_err_num:       3;                      /*The value of this signal means the number of error bytes.*/
-            uint32_t key4_fail:          1;                      /*0: Means no failure and that the data of key$n is reliable 1: Means that programming key$n failed and the number of error bytes is over 6.*/
+            uint32_t key3_fail:          1;                      /*0: Means no failure and that the data of key3 is reliable 1: Means that programming key3 failed and the number of error bytes is over 6.*/
         };
         uint32_t val;
     } rd_rs_err0;
     union {
         struct {
             uint32_t key5_err_num:      3;                       /*The value of this signal means the number of error bytes.*/
-            uint32_t key5_fail:         1;                       /*0: Means no failure and that the data of KEY5 is reliable 1: Means that programming user data failed and the number of error bytes is over 6.*/
+            uint32_t key4_fail:         1;                       /*0: Means no failure and that the data of KEY4 is reliable 1: Means that programming KEY4 failed and the number of error bytes is over 6.*/
             uint32_t sys_part2_err_num: 3;                       /*The value of this signal means the number of error bytes.*/
-            uint32_t sys_part2_fail:    1;                       /*0: Means no failure and that the data of system part2 is reliable 1: Means that programming user data failed and the number of error bytes is over 6.*/
+            uint32_t key5_fail:         1;                       /*0: Means no failure and that the data of KEY5 is reliable 1: Means that programming KEY5 failed and the number of error bytes is over 6.*/
             uint32_t reserved8:        24;                       /*Reserved.*/
         };
         uint32_t val;
