@@ -1,16 +1,8 @@
-// Copyright 2015-2020 Espressif Systems (Shanghai) PTE LTD
-//
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//     http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+/*
+ * SPDX-FileCopyrightText: 2015-2021 Espressif Systems (Shanghai) CO LTD
+ *
+ * SPDX-License-Identifier: Apache-2.0
+ */
 
 #ifndef __ESP_VFS_H__
 #define __ESP_VFS_H__
@@ -203,6 +195,10 @@ typedef struct
         int (*truncate_p)(void* ctx, const char *path, off_t length);                               /*!< truncate with context pointer */
         int (*truncate)(const char *path, off_t length);                                            /*!< truncate without context pointer */
     };
+    union {
+        int (*ftruncate_p)(void* ctx, int fd, off_t length);                                        /*!< ftruncate with context pointer */
+        int (*ftruncate)(int fd, off_t length);                                                     /*!< ftruncate without context pointer */
+    };
     union {
         int (*utime_p)(void* ctx, const char *path, const struct utimbuf *times);                   /*!< utime with context pointer */
         int (*utime)(const char *path, const struct utimbuf *times);                                /*!< utime without context pointer */
@@ -254,7 +250,6 @@ typedef struct
 #endif // CONFIG_VFS_SUPPORT_SELECT
 } esp_vfs_t;
 
-
 /**
  * Register a virtual filesystem for given path prefix.
  *
@@ -411,7 +406,8 @@ int esp_vfs_utime(const char *path, const struct utimbuf *times);
  * @param timeout   If not NULL, then points to timeval structure which
  *                  specifies the time period after which the functions should
  *                  time-out and return. If it is NULL, then the function will
- *                  not time-out.
+ *                  not time-out. Note that the timeout period is rounded up to
+ *                  the system tick and incremented by one.
  *
  * @return      The number of descriptors set in the descriptor sets, or -1
  *              when an error (specified by errno) have occurred.
