@@ -10,7 +10,7 @@ static const uint8_t LED_BUILTIN = 18;
 #define NUM_DIGITAL_PINS 40
 #define NUM_ANALOG_INPUTS 16
 
-#define analogInputToDigitalPin(p) (((p) < 20) ? (esp32_adc2gpio[(p)]) : -1)
+#define analogInputToDigitalPin(p) (((p) < 20) ? (analogChannelToDigitalPin(p)) : -1)
 #define digitalPinToInterrupt(p) (((p) < 40) ? (p) : -1)
 #define digitalPinHasPWM(p) (p < 34)
 
